==> iRRAM_2004_01.tar.gz: iRRAM_2004_01/src/convert.cc <==
/*

convert.cc -- a few conversion functions for the iRRAM library
 
Copyright (C) 2002/2003 Norbert Mueller / Tom van Diessen
 
This file is part of the iRRAM Library.
 
The iRRAM Library is free software; you can redistribute it and/or modify
it under the terms of the GNU Library General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at your
option) any later version.
 
The iRRAM Library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
License for more details.
 
You should have received a copy of the GNU Library General Public License
along with the iRRAM Library; see the file COPYING.LIB.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA 02111-1307, USA. 
*/

#include <math.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#include "iRRAM_core.h"

// conversion to type REAL from smaller types

REAL::REAL(const RATIONAL& r) : METRIC_OBJECT()
{
  INTEGER numi,deni;

  MP_rat_get_numerator(r.value,numi.value);
  MP_rat_get_denominator(r.value,deni.value);

  REAL result=REAL(numi)/REAL(deni);
  error=result.error;
  MP_duplicate_w_init(result.value,value);
}

REAL::REAL(const INTEGER& y): METRIC_OBJECT()
{
  MP_init(value);
  MP_INTEGER_to_mp(y.value,value);
  sizetype_exact(error);
}




INTEGER::INTEGER(const DYADIC& y)
{
	MP_int_init(value);
	MP_mp_to_INTEGER(y.value,value);
}

//Conversion from INTEGER to smaller types
INTEGER::operator long() const {;
  return MP_INTEGER_to_long(value);
}

INTEGER::operator int() const {;
  return MP_INTEGER_to_long(value);
}






//Conversions from REAL to smaller types
DYADIC::DYADIC(const REAL &x)
{ MP_init(value);
  (*this)=approx(x,DYADIC::getprec());
}

INTEGER::INTEGER(const REAL& x)
{
  sizetype psize,error;
  sizetype_set(psize,1,-4);
  if ( ! ACTUAL_STACK.inlimit) if (ikonvertanzahl > ikonvertaufrufe)
    { 
    MP_int_duplicate_wo_init(ikonvert[ikonvertaufrufe],value);
    ikonvertaufrufe++;
    return;
    }
  MP_int_init(value);
  REAL y;
  if (MP_sign(x.value)==1) y=x+REAL(0.5);
  else y=x-REAL(0.5);
  if (sizetype_less(psize,y.error) )
     {
       	DEBUG3(1,"insufficient precision %ld*2^(%ld) converting to integer\n",x.error.mantissa,x.error.exponent);
       	REITERATE(-y.error.exponent);
     }

  MP_mp_to_INTEGER(y.value,value);

  if ( ! ACTUAL_STACK.inlimit) {
    MP_int_duplicate_w_init(value,ikonvert[ikonvertaufrufe]);
    cacheinc_ikonvert();
    ikonvertaufrufe++;
    }
}


//Conversion from REAL to long
REAL::operator long() const {;
  return long(INTEGER(*this));
}

//Conversion from REAL to double
REAL::operator double() const {
  if ( bound(*this,-1150) ) return 0.0;
  long s=size(*this);
  DYADIC d=approx(*this,s-60);
  return MP_mp_to_double(d.value);
}

==> iRRAM_2004_02.tar.gz: iRRAM_2004_02/src/convert.cc <==
/*

convert.cc -- a few conversion functions for the iRRAM library
 
Copyright (C) 2002/2003 Norbert Mueller / Tom van Diessen
 
This file is part of the iRRAM Library.
 
The iRRAM Library is free software; you can redistribute it and/or modify
it under the terms of the GNU Library General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at your
option) any later version.
 
The iRRAM Library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
License for more details.
 
You should have received a copy of the GNU Library General Public License
along with the iRRAM Library; see the file COPYING.LIB.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA 02111-1307, USA. 
*/

#include <math.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#include "iRRAM_core.h"

#if defined(LRGMP)
	#include "LRGMP_ext.h"
	#define iRRAM_BACKEND "LRGMP"
#elif defined(cGMP)
	#include "cGMP_ext.h"
	#define iRRAM_BACKEND "cGMP"
#elif defined(GMP)
	#include "GMP_ext.h"
	#define iRRAM_BACKEND "GMP"
#elif defined(MPFR)
	#include "MPFR_ext.h"
	#define iRRAM_BACKEND "MPFR"
#elif defined(INTMPF)
	#include "INTMPF_ext.h"
	#define iRRAM_BACKEND "INTMPF"
#else
	#include "Backend not defined"
#endif

// conversion to type REAL from smaller types

REAL::REAL(const RATIONAL& r)
{
  INTEGER numi,deni;

  MP_rat_get_numerator(r.value,numi.value);
  MP_rat_get_denominator(r.value,deni.value);

  REAL result=REAL(numi)/REAL(deni);
  error=result.error;
  MP_duplicate_w_init(result.value,value);
  MP_getsize(value,vsize);
}

REAL::REAL(const INTEGER& y)
{
  MP_init(value);
  MP_INTEGER_to_mp(y.value,value);
  sizetype_exact(error);
  MP_getsize(value,vsize);
}




INTEGER::INTEGER(const DYADIC& y)
{
	MP_int_init(value);
	MP_mp_to_INTEGER(y.value,value);
}

//Conversion from INTEGER to smaller types
INTEGER::operator int() const {;
  return MP_INTEGER_to_int(value);
}





//Conversions from REAL to smaller types
DYADIC::DYADIC(const REAL &x)
{ MP_init(value);
  (*this)=approx(x,DYADIC::getprec());
}

INTEGER::INTEGER(const REAL& x)
{
  MP_int_type result;
  if ( (ACTUAL_STACK.inlimit==0) && cache_mpi.get(result)){
    MP_int_duplicate_w_init(result,value);
    return;
  }
  
  sizetype psize,error;
  sizetype_set(psize,1,-4);
  REAL y;
  if (MP_sign(x.value)==1) y=x+REAL(0.5);
  else y=x-REAL(0.5);
  if (sizetype_less(psize,y.error) )
     {
       	DEBUG3(1,"insufficient precision %ld*2^(%ld) converting to integer\n",x.error.mantissa,x.error.exponent);
       	REITERATE(-y.error.exponent);
     }
  MP_int_init(value);
  MP_mp_to_INTEGER(y.value,value);

  if ( ACTUAL_STACK.inlimit==0 ) {
    MP_int_duplicate_w_init(value,result);
    cache_mpi.put(result);
  }
  return;
}


//Conversion from REAL to int
REAL::operator int() const {;
  return int(INTEGER(*this));
}

//Conversion from REAL to double
REAL::operator double() const {
  if ( bound(*this,-1150) ) return 0.0;
  int s=size(*this);
  DYADIC d=approx(*this,s-60);
  return MP_mp_to_double(d.value);
}
